"use strict";

var _defaultShouldAsyncValidate = _interopRequireDefault(require("../defaultShouldAsyncValidate"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('defaultShouldAsyncValidate', function () {
  it('should not async validate if sync validation is not passing', function () {
    expect((0, _defaultShouldAsyncValidate.default)({
      syncValidationPasses: false
    })).toBe(false);
  });
  it('should async validate if blur triggered and sync passes', function () {
    expect((0, _defaultShouldAsyncValidate.default)({
      syncValidationPasses: true,
      trigger: 'blur'
    })).toBe(true);
  });
  it('should not async validate when pristine and initialized', function () {
    expect((0, _defaultShouldAsyncValidate.default)({
      syncValidationPasses: true,
      trigger: 'submit',
      pristine: true,
      initialized: true
    })).toBe(false);
  });
  it('should async validate when submitting and dirty', function () {
    expect((0, _defaultShouldAsyncValidate.default)({
      syncValidationPasses: true,
      trigger: 'submit',
      pristine: false,
      initialized: true
    })).toBe(true);
  });
  it('should async validate when submitting and not initialized', function () {
    expect((0, _defaultShouldAsyncValidate.default)({
      syncValidationPasses: true,
      trigger: 'submit',
      pristine: true,
      initialized: false
    })).toBe(true);
  });
  it('should not async validate when unknown trigger', function () {
    expect((0, _defaultShouldAsyncValidate.default)({
      syncValidationPasses: true,
      trigger: 'wtf'
    })).toBe(false);
  });
});