"use strict";

var _expect = _interopRequireDefault(require("expect"));

var _plain = _interopRequireDefault(require("../structure/plain"));

var _immutable = _interopRequireDefault(require("../structure/immutable"));

var _defaultShouldError = _interopRequireDefault(require("../defaultShouldError"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('defaultShouldError', function () {
  it('should validate when initialRender is true', function () {
    (0, _expect.default)((0, _defaultShouldError.default)({
      initialRender: true
    })).toBe(true);
  });

  var describeDefaultShouldError = function describeDefaultShouldError(structure) {
    var fromJS = structure.fromJS;
    it('should validate if values have changed', function () {
      (0, _expect.default)((0, _defaultShouldError.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooChanged'
          })
        }
      })).toBe(true);
    });
    it('should not validate if values have not changed', function () {
      (0, _expect.default)((0, _defaultShouldError.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooInitial'
          })
        }
      })).toBe(false);
    });
    it('should validate if field validator keys have changed', function () {
      (0, _expect.default)((0, _defaultShouldError.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooValue'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooValue'
          })
        },
        lastFieldValidatorKeys: [],
        fieldValidatorKeys: ['foo']
      })).toBe(true);
    });
    it('should not validate if field validator keys have not changed', function () {
      (0, _expect.default)((0, _defaultShouldError.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooInitial'
          })
        },
        lastFieldValidatorKeys: ['foo'],
        fieldValidatorKeys: ['foo']
      })).toBe(false);
    });
  };

  describeDefaultShouldError(_plain.default);
  describeDefaultShouldError(_immutable.default);
});