"use strict";

var _plain = _interopRequireDefault(require("../structure/plain"));

var _immutable = _interopRequireDefault(require("../structure/immutable"));

var _defaultShouldValidate = _interopRequireDefault(require("../defaultShouldValidate"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('defaultShouldValidate', function () {
  it('should validate when initialRender is true', function () {
    expect((0, _defaultShouldValidate.default)({
      initialRender: true
    })).toBe(true);
  });

  var describeDefaultShouldValidate = function describeDefaultShouldValidate(structure) {
    var fromJS = structure.fromJS;
    it('should validate if values have changed', function () {
      expect((0, _defaultShouldValidate.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooChanged'
          })
        }
      })).toBe(true);
    });
    it('should not validate if values have not changed', function () {
      expect((0, _defaultShouldValidate.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooInitial'
          })
        }
      })).toBe(false);
    });
    it('should validate if field validator keys have changed', function () {
      expect((0, _defaultShouldValidate.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooValue'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooValue'
          })
        },
        lastFieldValidatorKeys: [],
        fieldValidatorKeys: ['foo']
      })).toBe(true);
    });
    it('should not validate if field validator keys have not changed', function () {
      expect((0, _defaultShouldValidate.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooInitial'
          })
        },
        lastFieldValidatorKeys: ['foo'],
        fieldValidatorKeys: ['foo']
      })).toBe(false);
    });
  };

  describeDefaultShouldValidate(_plain.default);
  describeDefaultShouldValidate(_immutable.default);
});