"use strict";

var _expect = _interopRequireDefault(require("expect"));

var _plain = _interopRequireDefault(require("../structure/plain"));

var _immutable = _interopRequireDefault(require("../structure/immutable"));

var _defaultShouldWarn = _interopRequireDefault(require("../defaultShouldWarn"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('defaultShouldWarn', function () {
  it('should warn when initialRender is true', function () {
    (0, _expect.default)((0, _defaultShouldWarn.default)({
      initialRender: true
    })).toBe(true);
  });

  var describeDefaultShouldWarn = function describeDefaultShouldWarn(structure) {
    var fromJS = structure.fromJS;
    it('should warn if values have changed', function () {
      (0, _expect.default)((0, _defaultShouldWarn.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooChanged'
          })
        }
      })).toBe(true);
    });
    it('should not warn if values have not changed', function () {
      (0, _expect.default)((0, _defaultShouldWarn.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooInitial'
          })
        }
      })).toBe(false);
    });
    it('should warn if field validator keys have changed', function () {
      (0, _expect.default)((0, _defaultShouldWarn.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooValue'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooValue'
          })
        },
        lastFieldValidatorKeys: [],
        fieldValidatorKeys: ['foo']
      })).toBe(true);
    });
    it('should not warn if field validator keys have not changed', function () {
      (0, _expect.default)((0, _defaultShouldWarn.default)({
        initialRender: false,
        structure: structure,
        values: fromJS({
          foo: 'fooInitial'
        }),
        nextProps: {
          values: fromJS({
            foo: 'fooInitial'
          })
        },
        lastFieldValidatorKeys: ['foo'],
        fieldValidatorKeys: ['foo']
      })).toBe(false);
    });
  };

  describeDefaultShouldWarn(_plain.default);
  describeDefaultShouldWarn(_immutable.default);
});