"use strict";

var _getFormMeta = _interopRequireDefault(require("../getFormMeta"));

var _plain = _interopRequireDefault(require("../../structure/plain"));

var _expectations = _interopRequireDefault(require("../../structure/plain/__tests__/expectations"));

var _immutable = _interopRequireDefault(require("../../structure/immutable"));

var _expectations2 = _interopRequireDefault(require("../../structure/immutable/__tests__/expectations"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var describeGetFormMeta = function describeGetFormMeta(name, structure, setup) {
  var getFormMeta = (0, _getFormMeta.default)(structure);
  var fromJS = structure.fromJS,
      getIn = structure.getIn;
  describe(name, function () {
    beforeAll(function () {
      setup();
    });
    it('should return a function', function () {
      expect(typeof (0, _getFormMeta.default)('foo')).toBe('function');
    });
    it('should get the form values from state', function () {
      expect(getFormMeta('foo')(fromJS({
        form: {
          foo: {
            fields: {
              dog: {
                visited: true,
                touched: false
              },
              cat: {
                visited: false,
                touched: true
              }
            }
          }
        }
      }))).toEqualMap({
        dog: {
          visited: true,
          touched: false
        },
        cat: {
          visited: false,
          touched: true
        }
      });
    });
    it('should return object if there are no fields', function () {
      expect(getFormMeta('foo')(fromJS({
        form: {
          foo: {}
        }
      }))).toEqual(fromJS({}));
    });
    it('should use getFormState if provided', function () {
      expect(getFormMeta('foo', function (state) {
        return getIn(state, 'someOtherSlice');
      })(fromJS({
        someOtherSlice: {
          foo: {
            fields: {
              dog: {
                visited: true,
                touched: false
              },
              cat: {
                visited: false,
                touched: true
              }
            }
          }
        }
      }))).toEqualMap({
        dog: {
          visited: true,
          touched: false
        },
        cat: {
          visited: false,
          touched: true
        }
      });
    });
  });
};

describeGetFormMeta('getFormMeta.plain', _plain.default, function () {
  return expect.extend(_expectations.default);
});
describeGetFormMeta('getFormMeta.immutable', _immutable.default, function () {
  return expect.extend(_expectations2.default);
});