"use strict";

var _getFormSyncWarnings = _interopRequireDefault(require("../getFormSyncWarnings"));

var _plain = _interopRequireDefault(require("../../structure/plain"));

var _expectations = _interopRequireDefault(require("../../structure/plain/__tests__/expectations"));

var _immutable = _interopRequireDefault(require("../../structure/immutable"));

var _expectations2 = _interopRequireDefault(require("../../structure/immutable/__tests__/expectations"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var describeGetFormSyncErrors = function describeGetFormSyncErrors(name, structure, setup) {
  var getFormSyncWarnings = (0, _getFormSyncWarnings.default)(structure);
  var fromJS = structure.fromJS,
      getIn = structure.getIn;
  describe(name, function () {
    beforeAll(function () {
      setup();
    });
    it('should return a function', function () {
      expect(typeof (0, _getFormSyncWarnings.default)('foo')).toBe('function');
    });
    it('should get the form values from state', function () {
      expect(getFormSyncWarnings('foo')(fromJS({
        form: {
          foo: {
            syncWarnings: {
              dog: 'Snoopy',
              cat: 'Garfield'
            }
          }
        }
      }))).toEqualMap({
        dog: 'Snoopy',
        cat: 'Garfield'
      });
    });
    it('should return object if there are no syncWarnings', function () {
      expect(getFormSyncWarnings('foo')(fromJS({
        form: {
          foo: {}
        }
      }))).toEqual(fromJS({}));
    });
    it('should use getFormState if provided', function () {
      expect(getFormSyncWarnings('foo', function (state) {
        return getIn(state, 'someOtherSlice');
      })(fromJS({
        someOtherSlice: {
          foo: {
            syncWarnings: {
              dog: 'Snoopy',
              cat: 'Garfield'
            }
          }
        }
      }))).toEqualMap({
        dog: 'Snoopy',
        cat: 'Garfield'
      });
    });
  });
};

describeGetFormSyncErrors('getFormSyncWarnings.plain', _plain.default, function () {
  return expect.extend(_expectations.default);
});
describeGetFormSyncErrors('getFormSyncWarnings.immutable', _immutable.default, function () {
  return expect.extend(_expectations2.default);
});